/*
 * @(#)ContactEventTypesXmlSerializer.java
 */
package com.maileon.api.blacklists;

import com.maileon.api.XmlUtils;
import com.maileon.api.utils.MimeTypes;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

import javax.ws.rs.Consumes;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

@Provider
@Consumes(MimeTypes.MAILEON_API_XML)
@Produces(MimeTypes.MAILEON_API_XML)
public class BlacklistXmlSerializer implements MessageBodyReader<Blacklist>, MessageBodyWriter<Blacklist> {

    private static final String BLACKLIST_ELEMENT_NAME = "blacklist";
    private static final String ID_ELEMENT_NAME = "id";
    private static final String NAME_ELEMENT_NAME = "name";
    private static final String ENTRIES_ELEMENT_NAME = "entries";
    private static final String ENTRY_ELEMENT_NAME = "entry";

    @Override
    public boolean isReadable(Class<?> type, Type type1, Annotation[] antns, MediaType mt) {
        return type1 == Blacklist.class;
    }

    @Override
    public Blacklist readFrom(Class<Blacklist> type, Type type1, Annotation[] antns,
            MediaType mt, MultivaluedMap<String, String> mm, InputStream in) throws IOException,
            WebApplicationException {
        try {
            Element filters = XmlUtils.parseXmlDocument(in);
            return deserialize(filters);
        } catch (Throwable e) {
            throw new WebApplicationException(e);
        }
    }

    public static Blacklist deserialize(Element blacklist) {
        Blacklist result = new Blacklist();
        if (BLACKLIST_ELEMENT_NAME.equals(blacklist.getName())) {
            Element idElement = blacklist.element(ID_ELEMENT_NAME);
            if (idElement != null) {
                try {
                    result.setId(Long.parseLong(idElement.getText()));
                } catch (NumberFormatException e) {
                }
            }
            Element name = blacklist.element(NAME_ELEMENT_NAME);
            result.setName(name.getText());
            
            Element entriesElement = blacklist.element(ENTRIES_ELEMENT_NAME);
            if (entriesElement != null) {
                List<String> entries = new ArrayList<>();
                for (@SuppressWarnings("rawtypes") Iterator iterator = entriesElement.elementIterator(); iterator.hasNext();) {
                    Element entryElement = (Element) iterator.next();
                    if (ENTRY_ELEMENT_NAME.equals(entryElement.getName())) {
                        entries.add(entryElement.getText());
                    }
                }
                result.setEntries(entries);
            }
        }
        return result;
    }

    @Override
    public boolean isWriteable(Class<?> type, Type type1, Annotation[] antns, MediaType mt) {
        return type1 == Blacklist.class;
    }

    @Override
    public void writeTo(Blacklist t, Class<?> type, Type type1, Annotation[] antns, MediaType mt,
            MultivaluedMap<String, Object> mm, OutputStream out) throws IOException, WebApplicationException {
        if (t == null) {
            throw new WebApplicationException();
        }

        Element root;
        try {
            root = serialize(t);
            XMLWriter writer = new XMLWriter(out, OutputFormat.createPrettyPrint());
            writer.write(root);
            writer.close();
        } catch (Exception e) {
            throw new WebApplicationException(e);
        }

    }

    public static Element serialize(Blacklist blacklist) {
        Element blacklistElement = DocumentHelper.createElement(BLACKLIST_ELEMENT_NAME);

        if (blacklist.getId() != null) {
            XmlUtils.appendChildElementSafe(blacklistElement, ID_ELEMENT_NAME).setText(blacklist.getId().toString());
        }
        
        if (blacklist.getName() != null) {
            XmlUtils.appendChildElementSafe(blacklistElement, NAME_ELEMENT_NAME).setText(blacklist.getName());
        }
        
        if (blacklist.getEntries() != null) {
            Element entriesElement = XmlUtils.appendChildElementSafe(blacklistElement, ENTRIES_ELEMENT_NAME);
            for (String entry: blacklist.getEntries()) {
                Element entryElement = DocumentHelper.createElement(ENTRY_ELEMENT_NAME);
                entryElement.setText(entry);
                entriesElement.add(entryElement);
            }
        }
        
       return blacklistElement;
    }

    @Override
    public long getSize(Blacklist t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return -1;
    }

}
