package com.maileon.api.reports;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;

import com.maileon.api.MaileonException;
import com.maileon.api.Page;
import com.maileon.api.contacts.StandardContactField;
import com.maileon.api.test.TestProperties;
import org.junit.BeforeClass;
import org.junit.Test;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author Igor Schuraev
 */
public class MaileonReportsServiceTest {

    private static TestProperties testProperties;
    private static MaileonReportsService reportService;

    @BeforeClass
    public static void setUp() throws MaileonException {
        testProperties = new TestProperties();
        reportService = new MaileonReportsService(testProperties.getMaileonConfiguration());
        reportService.setDebug(testProperties.isDebug());
    }

    /**
     * Test of getOpens method, of class MaileonReportsService.
     */
    @Test
    public void testGetOpens() throws Exception {
        System.out.println("getOpens");
        Long fromDate = null;
        Long toDate = null;
        List<Long> mailingIdFilter = new ArrayList<>();
       // mailingIdFilter.add((long)200);

        List<Long> contactIdsFilter = new ArrayList<>();
        contactIdsFilter.add(50964l);
        contactIdsFilter.add(50998l);
        List<String> contactEmailsFilter = new ArrayList<>();
        contactEmailsFilter.add("marcus.staender@trusted-technologies.de");
        List<String> contactExternalIdsFilter = new ArrayList<>();
        contactExternalIdsFilter.add("1337");

        Format formatFilter = Format.HTML;
        List<String> socialNetworkFilter = null;
        List<String> deviceTypeFilter = null;
        boolean embedEmailClientInfos = true;
        boolean excludeAnonymousOpens = true;
        List<StandardContactField> standardFields = null;
        List<String> customFields = null;
        boolean embedFieldBackups = true;
        int pageIndex = 1;
        int pageSize = 10;
        Page<Open> result = reportService.getOpens(fromDate, toDate, mailingIdFilter, contactIdsFilter, contactEmailsFilter, contactExternalIdsFilter, formatFilter, socialNetworkFilter,
                deviceTypeFilter, embedEmailClientInfos, excludeAnonymousOpens, standardFields, customFields,
                embedFieldBackups, pageIndex, pageSize);
        System.out.println(result.getItems());
    }

    /**
     * Test of getOpens method, of class MaileonReportsService.
     */
    @Test
    public void testGetUniqueOpens() throws Exception {
        System.out.println("getOpens");
        Long fromDate = null;
        Long toDate = null;
        List<Long> mailingIdFilter = new ArrayList<>();
        // mailingIdFilter.add((long)200);

        List<Long> contactIdsFilter = new ArrayList<>();
        List<String> contactEmailsFilter = new ArrayList<>();
        //contactEmailsFilter.add("marcus.staender@trusted-technologies.de");
        List<String> contactExternalIdsFilter = new ArrayList<>();
        //contactExternalIdsFilter.add("1337");

        boolean embedEmailClientInfos = false;
        boolean excludeAnonymousOpens = true;
        List<StandardContactField> standardFields = null;
        List<String> customFields = null;
        boolean embedFieldBackups = false;
        int pageIndex = 1;
        int pageSize = 10;
        Page<Open> result = reportService.getUniqueOpens(fromDate, toDate, mailingIdFilter, contactIdsFilter, contactEmailsFilter, contactExternalIdsFilter,
                embedEmailClientInfos, excludeAnonymousOpens, standardFields, customFields,
                embedFieldBackups, pageIndex, pageSize);
        System.out.println(result.getItems());
    }

    /**
     * Test of getRecipients method, of class MaileonReportsService.
     */

    @Test
    public void testGetRecipients() throws Exception {
        System.out.println("getRecipients");
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Long fromDate = null;//df.parse("2014-07-30 00:00:00").getTime();
        Long toDate = null; //df.parse("2014-07-30 23:59:59").getTime();
        List<Long> mailingIdFilter = new ArrayList<>();
//        mailingIdFilter.add(74l);

        boolean excludeDeletedRecipients = true;
//        mailingIdFilter.add((long)200);
//        mailingIdFilter.add((long)116);

        List<Long> contactIdsFilter = null; //new ArrayList<Long>();
//        contactIdsFilter.add(40270l);
//        contactIdsFilter.add(40271l);
        List<String> contactEmailsFilter = new ArrayList<>();
        contactEmailsFilter.add("100@baunzt.de");
        List<String> contactExternalIdsFilter = null; //new ArrayList<String>();
//        contactExternalIdsFilter.add("100pro");

        List<StandardContactField> standardFields = new ArrayList <>();
        List<String> customFields = new ArrayList<>();
        standardFields.add(StandardContactField.FULLNAME);
        standardFields.add(StandardContactField.BIRTHDAY);
        boolean embedFieldBackups = false;
        int pageIndex = 1;
        int pageSize = 10;
        Page<Recipient> result = reportService.getRecipients(fromDate, toDate, mailingIdFilter, contactIdsFilter, contactEmailsFilter, contactExternalIdsFilter, excludeDeletedRecipients,
                standardFields, customFields, embedFieldBackups, pageIndex, pageSize);
        System.out.println("Total items: " + result.getTotalItems());
        System.out.println("Total pages: " + result.getCountPages());
        System.out.println("Items size: " + result.getItems().size());
        System.out.println(result.getItems());
    }

    @Test
    public void testGetRecipientsCount() throws Exception {
        System.out.println("getRecipientsCount");
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Long fromDate = df.parse("2014-07-25 17:00:00").getTime();
        Long toDate = df.parse("2014-07-29 17:00:59").getTime();
//        Long fromDate = df.parse("2014-06-17 00:00:00").getTime();
//        Long toDate = df.parse("2014-06-17 23:59:59").getTime();
        List<Long> mailingIdFilter = new ArrayList<>();
//        mailingIdFilter.add(74l);
        //mailingIdFilter.add(238l);

        boolean excludeDeletedRecipients = false;
//        mailingIdFilter.add((long)200);
//        mailingIdFilter.add((long)116);

        List<Long> contactIdsFilter = null; //new ArrayList<Long>();
//        contactIdsFilter.add(40270l);
//        contactIdsFilter.add(40271l);
        List<String> contactEmailsFilter = null; //new ArrayList<String>();
//        contactEmailsFilter.add("100@baunzt.de");
        List<String> contactExternalIdsFilter = null; //new ArrayList<String>();
//        contactExternalIdsFilter.add("100pro");


       int result = reportService.getRecipientsCount(fromDate, toDate, mailingIdFilter, contactIdsFilter, contactEmailsFilter, contactExternalIdsFilter, excludeDeletedRecipients);
        System.out.println("Total items: " + result);
    }

    /**
     * Test of getClicks method, of class MaileonReportsService.
     */
    @Test
    public void testGetClicks() throws Exception {
        System.out.println("getClicks");
        Long fromDate = null;
        Long toDate = null;
        List<Long> mailingIdFilter = new ArrayList<>();
        mailingIdFilter.add(238l);

        List<Long> contactIdsFilter = null; //new ArrayList<Long>();
//        contactIdsFilter.add(40270l);
//        contactIdsFilter.add(40271l);
        List<String> contactEmailsFilter = new ArrayList<>();
        contactEmailsFilter.add("marcus.staender@trusted-technologies.de");
        List<String> contactExternalIdsFilter = null; //new ArrayList<String>();
//        contactExternalIdsFilter.add("100pro");

        Format formatFilter = null;
        List<Long> linkIdFilter = null;
        String linkUrlFilter = "";
        List<String> linkTagFilter = null;
        List<String> socialNetworkFilter = null;
        List<String> deviceTypeFilter = null;
        boolean embedEmailClientInfos = false;
        boolean excludeAnonymousClicks = false;
        List<StandardContactField> standardFields = null;
        List<String> customFields = null;
        boolean embedFieldBackups = false;
        int pageIndex = 1;
        int pageSize = 10;
        Page<Click> result = reportService.getClicks(fromDate, toDate, mailingIdFilter, contactIdsFilter, contactEmailsFilter, contactExternalIdsFilter, formatFilter, linkIdFilter, linkUrlFilter,
                linkTagFilter, socialNetworkFilter, deviceTypeFilter, embedEmailClientInfos, excludeAnonymousClicks,
                standardFields, customFields, embedFieldBackups, pageIndex, pageSize);
        System.out.println(result.getItems());
    }

    /**
     * Test of getBounces method, of class MaileonReportsService.
     */
    @Test
    public void testGetBounces() throws Exception {
        System.out.println("getBounces");
        Long fromDate = null;
        Long toDate = null;
        List<Long> mailingIdFilter = new ArrayList<>();
        mailingIdFilter.add(238l);

        List<Long> contactIdsFilter = null; //new ArrayList<Long>();
//        contactIdsFilter.add(40270l);
//        contactIdsFilter.add(40271l);
        List<String> contactEmailsFilter = null; //new ArrayList<String>();
//        contactEmailsFilter.add("100@baunzt.de");
        List<String> contactExternalIdsFilter = null; //new ArrayList<String>();
//        contactExternalIdsFilter.add("100pro");

        List<String> statusCodeFilter = null;
        String typeFilter = "";
        String sourceFilter = "";
        boolean excludeAnonymousBounces = false;
        List<StandardContactField> standardFields = null;
        List<String> customFields = null;
        boolean embedFieldBackups = false;
        int pageIndex = 1;
        int pageSize = 10;
        Page<Bounce> result = reportService.getBounces(fromDate, toDate, mailingIdFilter, contactIdsFilter, contactEmailsFilter, contactExternalIdsFilter, statusCodeFilter, typeFilter,
                sourceFilter, excludeAnonymousBounces, standardFields, customFields, embedFieldBackups, pageIndex,
                pageSize);
        System.out.println(result.getItems());
    }

    /**
     * Test of getUnsubscriptions method, of class MaileonReportsService.
     */
    @Test
    public void testGetUnsubscriptions() throws Exception {
        System.out.println("getUnsubscriptions");
        Long fromDate = null;
        Long toDate = null;
        List<Long> mailingIdFilter = new ArrayList<>();
//        mailingIdFilter.add(238l);

        List<Long> contactIdsFilter = null; //new ArrayList<Long>();
//        contactIdsFilter.add(40270l);
//        contactIdsFilter.add(40271l);
        List<String> contactEmailsFilter = new ArrayList<>();
        contactEmailsFilter.add("lisa.stecher@web.de");
        List<String> contactExternalIdsFilter = new ArrayList<>();
//        contactExternalIdsFilter.add("ext");

        String sourceFilter = "";
        boolean embedFieldBackups = false;
        int pageIndex = 1;
        int pageSize = 100;
        Page<Unsubscriber> result = reportService.getUnsubscribers(fromDate, toDate, mailingIdFilter, contactIdsFilter, contactEmailsFilter, contactExternalIdsFilter, sourceFilter, embedFieldBackups,
                pageIndex, pageSize);
//        System.out.println(result.getItems());
    }

    @Test
    public void testGetSubscribers() throws Exception {

        List<Long> mailingIdFilter = new ArrayList<>();
        mailingIdFilter.add(238l);

        List<Long> contactIdsFilter = null;
        List<String> contactEmailsFilter = null;
        List<String> contactExternalIdsFilter = null;

        @SuppressWarnings("deprecation")
        Page<Subscriber> result = reportService.getSubscribers(
                null, //new Timestamp(2014 - 1900, 3 - 1, 24, 00, 00, 00, 00).getTime(),
                null, //new Timestamp(2014 - 1900, 3 - 1, 24, 00, 00, 00, 00).getTime(),
                mailingIdFilter, contactIdsFilter, contactEmailsFilter, contactExternalIdsFilter, false, Arrays.asList(StandardContactField.TITLE), Arrays.asList("Dorig"), false, 1, 100);
        assertThat(result.getItems(), contains(hasProperty("contact", allOf(
                        hasProperty("email", is("jannik.jochem@trusted-mails.com")),
                        hasProperty("standardFields", hasEntry(StandardContactField.TITLE, "Freigraf")),
                        hasProperty("customFields", hasEntry("Dorig", "1"))))));
    }

    @Test
    public void testGetSubscribersExcludeAnonymous() throws Exception {

        List<Long> contactIdsFilter = null;
        List<String> contactEmailsFilter = null;
        List<String> contactExternalIdsFilter = null;

        @SuppressWarnings("deprecation")
        Page<Subscriber> resultWithAnonymousClicks = reportService.getSubscribers(
                new Timestamp(2010 - 1900, 3 - 1, 24, 00, 00, 00, 00).getTime(),
                new Timestamp(2013 - 1900, 12 - 1, 24, 00, 00, 00, 00).getTime(),
                null, contactIdsFilter, contactEmailsFilter, contactExternalIdsFilter, false, null, null, false, 1, 100);
        @SuppressWarnings("deprecation")
        Page<Subscriber> resultWithoutAnonymousClicks = reportService.getSubscribers(
                new Timestamp(2010 - 1900, 3 - 1, 24, 00, 00, 00, 00).getTime(),
                new Timestamp(2013 - 1900, 12 - 1, 24, 00, 00, 00, 00).getTime(),
                null, contactIdsFilter, contactEmailsFilter, contactExternalIdsFilter, true, null, null, false, 1, 100);
        assertThat(resultWithAnonymousClicks.getItems().size(), is(greaterThan(0)));
        assertThat(resultWithoutAnonymousClicks.getItems().size(), is(0));
    }
}