package com.maileon.api.transactions;

import com.maileon.api.MaileonException;
import com.maileon.api.contacts.*;
import com.maileon.api.test.TestProperties;
import org.junit.BeforeClass;
import org.junit.Test;

import java.util.HashMap;
import java.util.Map;

public class ContactFormTest {

    // The contact form is sent to this email address
    private static final String CF_MANAGER_EMAIL = "tarkan.corak@groz-beckert.com";

    // Contact form data
    private static final String CF_FIRSTNAME = "Max";
    private static final String CF_LASTNAME = "Mustermann";
    private static final String CF_EMAIL = "max.mustermann@xqueue.com";
    private static final String CF_TEXT = "Hello everyone, your page is so nice.";

    private static final long TRANSACTION_TYPE_ID = 8;


    // For creating the contact
    private static MaileonContactsService contactsService;

    // For sending the transaction
    private static MaileonTransactionsService transactionService;

    // Test properties to connect the services to the Maileon account
    private static TestProperties testProperties;
    

    @BeforeClass
    public static void setUp() throws MaileonException {
        testProperties = new TestProperties();
        transactionService = new MaileonTransactionsService(testProperties.getMaileonConfiguration());
        transactionService.setDebug(testProperties.isDebug());
        contactsService = new MaileonContactsService(testProperties.getMaileonConfiguration());
        contactsService.setDebug(testProperties.isDebug());

        // I do this in the initialization to ensure the manager contact really exists but it is not really
        // required due to using the import contact statement later when sending the transaction
        try {
            Contact contact= new Contact();
            contact.setAnonymous(false);
            contact.setEmail(CF_MANAGER_EMAIL);
            contact.setPermission(Permission.OTHER);
            contactsService.createContact(contact, SynchronizationMode.IGNORE);
        } catch (MaileonException e) {
            e.printStackTrace();
            throw e;
        }
    }

    @Test
    /**
     * We assume someone filled a contact form and now the request is sent to some case manager
     */
    public void createContactAndSendTransactionToManager() throws MaileonException {

        // Ok, let's first create the contact with permission "other" to allow Maileon send transactions when answering
        try {
            Contact contact= new Contact();
            contact.setAnonymous(false);
            contact.setEmail(CF_EMAIL);
            contact.setPermission(Permission.OTHER);
            contact.getStandardFields().put(StandardContactField.FIRSTNAME, CF_FIRSTNAME);
            contact.getStandardFields().put(StandardContactField.LASTNAME, CF_LASTNAME);
            contactsService.createContact(contact, SynchronizationMode.UPDATE);
        } catch (MaileonException e) {
            e.printStackTrace();
            throw e;
        }


        // Now send the "contact-transaction" to the manager
        try {
            // Create the transaction object
            Transaction transaction = new Transaction();
            transaction.setType(TRANSACTION_TYPE_ID);

            // Create the contact to whom the form is sent, use import reference if the contact might not exist
            ImportContactReference contact = new ImportContactReference();
            contact.setPermission(Permission.OTHER.getCode());
            contact.setEmail(CF_MANAGER_EMAIL);

            ImportReference contactReference = new ImportReference();
            contactReference.setContact(contact);
            transaction.setImportReference(contactReference);


            Map<String, String> content = new HashMap<>();
            content.put("firstname", CF_FIRSTNAME);
            content.put("lastname", CF_LASTNAME);
            content.put("email", CF_EMAIL);
            content.put("message", CF_TEXT);
            transaction.setContent(content);

            // Finally send the transaction
            TransactionProcessingReport report = transactionService.createTransaction(transaction, true);
        } catch (MaileonException e) {
            e.printStackTrace();
            throw e;
        }
    }
}
