package com.maileon.api.util;

import com.maileon.api.MaileonException;
import com.maileon.api.ping.MaileonPingService;
import com.maileon.api.test.TestProperties;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Test driver for the ping service
 * 
 * @author Felix Heinrichs | Trusted Mails GmbH | <a href="mailto:felix.heinrichs@trusted-mails.com">felix.heinrichs@trusted-mails.com</a>
 * @author Marcus St&auml;nder | Trusted Mails GmbH | <a href="mailto:marcus.staender@trusted-mails.com">marcus.staender@trusted-mails.com</a>
 *
 */
public class MaileonPingUtility {

    private static TestProperties testProperties;
    protected static MaileonPingService service = null;

    @BeforeClass
    public static void setUp() throws MaileonException {
        testProperties = new TestProperties();
        service = new MaileonPingService(testProperties.getMaileonConfiguration());
        service.setDebug(testProperties.isDebug());
    }
    
    /**
     * Check if the API-Key has retrieve rights
     */
    @Test
    public void testRetrieve() throws Throwable {
        try {
            int statusCode = service.checkRetrieve().getStatusCode();
            assert(200 <= statusCode && 299 >= statusCode);
        } catch (MaileonException e) {
            System.out.println(String.format("FAILED [%s - %s (%s)]", e.getStatusCode(), e.getReasonPhrase(), e.getMessage()));
            throw e;
        }
    }

    /**
     * Check if the API-Key has create rights
     */
    @Test
    public void testCreate() throws Throwable {
        try {
            int statusCode = service.checkCreate().getStatusCode();
            assert(200 <= statusCode && 299 >= statusCode);
        } catch (MaileonException e) {
            System.out.println(String.format("FAILED [%s - %s (%s)]", e.getStatusCode(), e.getReasonPhrase(), e.getMessage()));
            throw e;
        }
    }

    /**
     * Check if the API-Key has update rights
     */
    @Test
    public void testUpdate() throws Throwable {
        try {
            int statusCode = service.checkUpdate().getStatusCode();
            assert(200 <= statusCode && 299 >= statusCode);
        } catch (MaileonException e) {
            System.out.println(String.format("FAILED [%s - %s (%s)]", e.getStatusCode(), e.getReasonPhrase(), e.getMessage()));
            throw e;
        }
    }

    /**
     * Check if the API-Key has delete rights
     */
    @Test
    public void testDelete() throws Throwable {
        try {
            int statusCode = service.checkDelete().getStatusCode();
            assert(200 <= statusCode && 299 >= statusCode);
        } catch (MaileonException e) {
            System.out.println(String.format("FAILED [%s - %s (%s)]", e.getStatusCode(), e.getReasonPhrase(), e.getMessage()));
            throw e;
        }
    }
}
